package main

import (
	"fmt"
	"io"
	"net"
	"sync"
	"time"
)

type Server struct {
	Ip   string
	Port int

	OnlineMap map[string]*User
	mapLock   sync.RWMutex

	Message chan string
}

// 创建 server 的接口
func NewServer(ip string, port int) *Server {
	server := &Server{
		Ip:        ip,
		Port:      port,
		OnlineMap: make(map[string]*User),
		Message:   make(chan string),
	}

	return server
}

// 当监听到message广播消息,就将消息发送给全部的在线User
func (s *Server) ListenMessager() {
	for {
		msg := <-s.Message
		s.mapLock.Lock()
		for _, cli := range s.OnlineMap {
			cli.C <- msg
		}
		s.mapLock.Unlock()
	}
}

// 广播消息的方法
func (s *Server) BroadCast(user *User, msg string) {
	sendMsg := "[" + user.Addr + "]" + user.Name + ":" + msg
	s.Message <- sendMsg
}

// 处理请求
func (s *Server) Handler(conn net.Conn) {
	fmt.Println("链接建立成功！！！")
	user := NerUser(conn, s)
	////将当前用户插入map中
	//s.mapLock.Lock()
	//s.OnlineMap[user.Name] = user
	//s.mapLock.Unlock()
	////广播当前用户上线消息
	//s.BroadCast(user, "已上线")
	user.Online()

	isLive := make(chan bool)

	//处理客户端发送的消息，由于是堵塞状态，新开一个goroutine
	go func() {
		buf := make([]byte, 4096)
		for {
			n, err := conn.Read(buf)
			if n == 0 {
				user.Offline()
				//s.BroadCast(user, "下线")
				return
			}
			if err != nil && err != io.EOF {
				fmt.Println("Conn Error:", err)
				return
			}
			msg := string(buf[:n-1])
			//s.BroadCast(user, msg)
			user.DoMessage(msg)

			isLive <- true
		}
	}()

	for {
		select {
		case <-isLive:
		case <-time.After(time.Second * 60 * 30):
			user.sendMsg("你已超时，将被踢出下线")
			//关闭channel
			close(user.C)
			//关闭链接
			conn.Close()
			delete(s.OnlineMap, user.Name)
			return
		}
	}
}

// 启动服务器接口
func (s *Server) Start() {
	listener, err := net.Listen("tcp", fmt.Sprintf("%s:%d", s.Ip, s.Port))
	if err != nil {
		fmt.Println("net.Listen err:", err)
		return
	}
	fmt.Println("Server 运行成功~~~")
	defer listener.Close()

	go s.ListenMessager()

	for {
		//accept
		conn, err := listener.Accept()
		if err != nil {
			fmt.Println("listener accept err:", err)
			continue
		}
		//do handler
		go s.Handler(conn)
	}
}
