package main

import (
	"net"
	"strings"
)

type User struct {
	Name   string
	Addr   string
	C      chan string
	conn   net.Conn
	server *Server
}

// 创建用户的接口
func NerUser(conn net.Conn, server *Server) *User {
	userAddr := conn.RemoteAddr().String()
	user := &User{
		Name:   userAddr,
		Addr:   userAddr,
		C:      make(chan string),
		conn:   conn,
		server: server,
	}

	//启动当前user channel的goroutine
	go user.ListenMessage()

	return user
}

// 监听消息
func (u *User) ListenMessage() {
	for {
		msg := <-u.C
		u.conn.Write([]byte(msg + "\n"))
	}
}

func (u *User) Online() {
	//将当前用户插入map中
	u.server.mapLock.Lock()
	u.server.OnlineMap[u.Name] = u
	u.server.mapLock.Unlock()
	//广播当前用户上线消息
	u.server.BroadCast(u, "已上线")
}

func (u *User) Offline() {
	//将当前用户插入map中
	u.server.mapLock.Lock()
	delete(u.server.OnlineMap, u.Name)
	u.server.mapLock.Unlock()
	//广播当前用户上线消息
	u.server.BroadCast(u, "下线")
}

func (u *User) DoMessage(msg string) {
	if msg == "who" {
		u.server.mapLock.Lock()
		for _, user := range u.server.OnlineMap {
			onlineMsg := "[" + user.Name + "] is online\n"
			u.sendMsg(onlineMsg)
		}
		u.server.mapLock.Unlock()
	} else if len(msg) > 7 && msg[:7] == "rename|" {
		newName := strings.Split(msg, "|")[1]
		_, isExist := u.server.OnlineMap[newName]
		if isExist {
			u.sendMsg("用户名已存在，修改失败")
		} else {
			u.server.mapLock.Lock()
			oldName := u.Name
			u.server.OnlineMap[newName] = u
			delete(u.server.OnlineMap, oldName)
			u.Name = newName
			u.sendMsg("用户名修改成功,[" + oldName + "]->[" + newName + "]")
			u.server.mapLock.Unlock()
		}
	} else if len(msg) > 3 && msg[:3] == "to|" {
		targetUserName := strings.Split(msg, "|")[1]
		if targetUserName == "" {
			u.sendMsg("发送私聊消息格式错误,正确格式:to|张三|你好呀")
			return
		}
		targetUser, isExist := u.server.OnlineMap[targetUserName]
		if !isExist {
			u.sendMsg("目标对象不存在，请仔细检查")
			return
		}
		content := strings.Split(msg, "|")[2]
		if content == "" {
			u.sendMsg("消息内容格式错误")
			return
		}
		targetUser.sendMsg("来自:[" + u.Name + "]的消息，内容是:" + content)
	} else {
		u.server.BroadCast(u, msg)
	}
}

func (u *User) sendMsg(msg string) {
	u.conn.Write([]byte(msg))
}
